{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "[]\n"
     ]
    }
   ],
   "source": [
    "import requests\n",
    "import xml.etree.ElementTree as ET\n",
    "from pymarc import marcxml\n",
    "\n",
    "class OCLC_API(object):\n",
    "\n",
    "    def __init__(self, api_key, request_wait=0.5):\n",
    "        self.xisbn_url = \"http://xisbn.worldcat.org/webservices/xid/isbn/\"\n",
    "        self.xisbn_query = \"?method=getEditions&format=json&fl=*\"\n",
    "        self.search_url = \"http://www.worldcat.org/webservices/catalog/search/worldcat/sru\"\n",
    "        self.api_key = api_key\n",
    "        self.request_wait = request_wait\n",
    "\n",
    "    def make_author_title_query(self, author, title, record_schema=\"marcxml\"):\n",
    "        query = 'srw.au+all+\"%s\"+and+srw.ti+all+\"%s\"' % (author, title)\n",
    "        schema = \"info%%3Asrw%%2Fschema%%2F1%%2F%s\" % (record_schema)\n",
    "        return \"%s?query=%s&wskey=%s&recordSchema=%s\" % (self.search_url, query, self.api_key, schema)\n",
    "\n",
    "    def get_editions(self, isbn):\n",
    "        isbn_url = \"%s%s%s\" % (self.xisbn_url, isbn, self.xisbn_query)\n",
    "        response = requests.get(isbn_url)\n",
    "        if 'list' in response.json():\n",
    "            return response.json()['list']\n",
    "        return []\n",
    "\n",
    "    def map_isbns(self, isbns):\n",
    "        mapped = {}\n",
    "        for index, isbn in enumerate(isbns):\n",
    "            mapped[isbn] = self.get_editions(isbn)\n",
    "            print(index+1, \" mappings retrieved\")\n",
    "            time.sleep(self.request_wait)\n",
    "        return mapped\n",
    "\n",
    "    def find_work_records(self, author, title, record_schema=\"marcxml\"):\n",
    "        url = self.make_author_title_query(author, title)\n",
    "        response = requests.get(url)\n",
    "        root = ET.fromstring(response.text)\n",
    "        for child in root:\n",
    "            if child.tag == \"{http://www.loc.gov/zing/srw/}records\":\n",
    "                return child\n",
    "        return None\n",
    "\n",
    "    def get_isbns(self, records):\n",
    "        if isinstance(records, list):\n",
    "            return [self.get_isbn(record) for record in records]\n",
    "        else:\n",
    "            return []\n",
    "\n",
    "    def get_isbn(self, record):\n",
    "        temp_file = \"tempfile.xml\"\n",
    "        with open(temp_file, 'wb') as fh:\n",
    "            fh.write(ET.tostring(record))\n",
    "        reader = marcxml.parse_xml_to_array(temp_file)\n",
    "        for record in reader:\n",
    "            if record:\n",
    "                metadata = {\n",
    "                    \"author\": record.author(),\n",
    "                    \"title\": record.title(),\n",
    "                    \"isbn\": [field['a'] for field in record.get_fields('020')],\n",
    "                    \"notes\": [note['a'] for note in record.notes()],\n",
    "                    \"publisher\": record.publisher(),\n",
    "                    \"pubyear\": record.pubyear()\n",
    "                }\n",
    "                return metadata\n",
    "                return record.isbn()\n",
    "        return None\n",
    "\n",
    "    def find_work_metadata(self, author, title, record_schema=\"marcxml\"):\n",
    "        records = self.find_work_records(author, title)\n",
    "        return self.get_isbns(records)\n",
    "\n",
    "    \n",
    "def test_oclc_api():\n",
    "    oclc_api_key = \"5GymVhuhaTbrsUg0NSgTGEdH8OevLipG6LIGaZAsLqT5ib1MaaGMQTDKkJlr9Ywjh0ooUFcdcnL4RQX3\"\n",
    "    oclc = OCLC_API(oclc_api_key)\n",
    "    author = \"zadie smith\"\n",
    "    title = \"witte tanden\"\n",
    "    isbns = oclc.find_work_metadata(author, title)\n",
    "    print(isbns)\n",
    "\n",
    "\n",
    "test_oclc_api()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 50,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "leestafel.info/a-beck\n",
      "leestafel.info/a-berebrouckx\n",
      "leestafel.info/a-bohlmeijer\n",
      "leestafel.info/a-bon\n",
      "leestafel.info/a-chambers\n",
      "leestafel.info/a-coudenys-a-a-van-lierde\n",
      "leestafel.info/a-de-saint-exupery\n",
      "leestafel.info/a-desmond-a-j-moore\n",
      "leestafel.info/a-el-baraka\n",
      "leestafel.info/a-el-baraka?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/a-el-baraka?format=feed&type=atom\n",
      "leestafel.info/a-el-baraka?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/a-el-baraka?format=feed&type=rss\n",
      "leestafel.info/a-f-th-van-der-heijden\n",
      "leestafel.info/a-havukainen-a-s-toivonen\n",
      "leestafel.info/a-kranendonk\n",
      "leestafel.info/a-kruijssen\n",
      "leestafel.info/a-lindgren\n",
      "leestafel.info/a-lootens\n",
      "leestafel.info/a-machowiak-a-d-mizieliski\n",
      "leestafel.info/a-mg-schmidt\n",
      "leestafel.info/a-moravia-a-a-elkann\n",
      "leestafel.info/a-munoz-molina\n",
      "leestafel.info/a-sax\n",
      "leestafel.info/a-smids\n",
      "leestafel.info/a-steinhoefel\n",
      "leestafel.info/a-v-praag\n",
      "leestafel.info/a-van-dam\n",
      "leestafel.info/a-van-den-brink\n",
      "leestafel.info/a-van-gansewinkel\n",
      "leestafel.info/a-van-haeringen\n",
      "leestafel.info/a-van-heugten\n",
      "leestafel.info/a-van-praag\n",
      "leestafel.info/a-visser\n",
      "leestafel.info/a-west\n",
      "leestafel.info/a-woltz\n",
      "leestafel.info/aad-verhoef\n",
      "leestafel.info/aaf-brandt-corstius\n",
      "leestafel.info/aafke-steenhuis\n",
      "leestafel.info/aat-ceelen\n",
      "leestafel.info/abdelkader-benali-a-saida-nadi-benali\n",
      "leestafel.info/abhijit-v-banerjee-en-esther-duflo\n",
      "leestafel.info/achilles-cools\n",
      "leestafel.info/adam-foulds\n",
      "leestafel.info/adam-haslett\n",
      "leestafel.info/adam-stower\n",
      "leestafel.info/adam-sutherland\n",
      "leestafel.info/adri-treffers\n",
      "leestafel.info/adriaan-van-dis\n",
      "leestafel.info/adrian-goldsworthy\n",
      "leestafel.info/aesopus-a-ayano-imai\n",
      "leestafel.info/af-harrold\n",
      "leestafel.info/agave-kruijssen\n",
      "leestafel.info/aglaia-bouma\n",
      "leestafel.info/agnes-martin-lugand\n",
      "leestafel.info/agnes-van-den-muijsenberg\n",
      "leestafel.info/agnes-verbiest\n",
      "leestafel.info/agostino-traini\n",
      "leestafel.info/ahj-dautzenberg\n",
      "leestafel.info/ahj-dautzenberg-non-fictie\n",
      "leestafel.info/ai-mi\n",
      "leestafel.info/aidan-chambers\n",
      "leestafel.info/aj-betts\n",
      "leestafel.info/aj-roos\n",
      "leestafel.info/aje-boschhuizen\n",
      "leestafel.info/alaa-al-aswani\n",
      "leestafel.info/alaa-al-aswani?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alaa-al-aswani?format=feed&type=atom\n",
      "leestafel.info/alaa-al-aswani?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alaa-al-aswani?format=feed&type=rss\n",
      "leestafel.info/alain-de-botton\n",
      "leestafel.info/alain-grootaers\n",
      "leestafel.info/alan-bennett\n",
      "leestafel.info/alan-bennett?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alan-bennett?format=feed&type=atom\n",
      "leestafel.info/alan-bennett?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alan-bennett?format=feed&type=rss\n",
      "leestafel.info/alan-drew\n",
      "leestafel.info/alan-macdonald\n",
      "leestafel.info/alastair-fothergill\n",
      "leestafel.info/albert-camus\n",
      "leestafel.info/alberto-melis\n",
      "leestafel.info/alberto-moravia\n",
      "leestafel.info/aleid-truijens\n",
      "leestafel.info/alejandro-zambra\n",
      "leestafel.info/aleksandra-mizielinska-a-daniel-mizielinski\n",
      "leestafel.info/alessandro-baricco\n",
      "leestafel.info/alessandro-baricco?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alessandro-baricco?format=feed&type=atom\n",
      "leestafel.info/alessandro-baricco?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alessandro-baricco?format=feed&type=rss\n",
      "leestafel.info/alessandro-gatti\n",
      "leestafel.info/alex-bakker\n",
      "leestafel.info/alex-boogers\n",
      "leestafel.info/alex-boogers?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alex-boogers?format=feed&type=atom\n",
      "leestafel.info/alex-boogers?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alex-boogers?format=feed&type=rss\n",
      "leestafel.info/alex-gino\n",
      "leestafel.info/alex-haas\n",
      "leestafel.info/alex-van-heezik\n",
      "leestafel.info/alexander-mccall-smith\n",
      "leestafel.info/alexander-ramati\n",
      "leestafel.info/alexander-van-der-meer\n",
      "leestafel.info/alexandra-besel\n",
      "leestafel.info/alexandra-terlouw-van-hulst\n",
      "leestafel.info/alexandra-van-der-geer\n",
      "leestafel.info/alexandra-van-steen\n",
      "leestafel.info/alexandre-dumas-fils\n",
      "leestafel.info/alexis-deacon-a-viviane-schwarz\n",
      "leestafel.info/alfons-sluizer\n",
      "leestafel.info/alfred-boeve\n",
      "leestafel.info/ali-benjamin\n",
      "leestafel.info/ali-shaw\n",
      "leestafel.info/alice-briere-haquet-a-olivier-philipponneau\n",
      "leestafel.info/alice-laplante\n",
      "leestafel.info/alice-munro\n",
      "leestafel.info/alice-sebold\n",
      "leestafel.info/alicia-erian\n",
      "leestafel.info/aliefka-bijlsma\n",
      "leestafel.info/aliefka-bijlsma?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/aliefka-bijlsma?format=feed&type=atom\n",
      "leestafel.info/aliefka-bijlsma?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/aliefka-bijlsma?format=feed&type=rss\n",
      "leestafel.info/alina-nube\n",
      "leestafel.info/alison-goodman\n",
      "leestafel.info/alison-wearing\n",
      "leestafel.info/alissa-york\n",
      "leestafel.info/allan-ahlberg\n",
      "leestafel.info/allard-schroeder\n",
      "leestafel.info/allen-zadoff\n",
      "leestafel.info/alstein\n",
      "leestafel.info/alstein?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alstein?format=feed&type=atom\n",
      "leestafel.info/alstein?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/alstein?format=feed&type=rss\n",
      "leestafel.info/alston-w-purvis-ea\n",
      "leestafel.info/aly-hilberts\n",
      "leestafel.info/alyson-richman\n",
      "leestafel.info/alyze-bos\n",
      "leestafel.info/amanda-berry-a-gina-dejesus\n",
      "leestafel.info/amanda-howells\n",
      "leestafel.info/amelia-freer\n",
      "leestafel.info/amelie-nothomb\n",
      "leestafel.info/aminatta-forna\n",
      "leestafel.info/amor-towles\n",
      "leestafel.info/amy-bloom\n",
      "leestafel.info/amy-bloom?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/amy-bloom?format=feed&type=atom\n",
      "leestafel.info/amy-bloom?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/amy-bloom?format=feed&type=rss\n",
      "leestafel.info/amy-chua\n",
      "leestafel.info/amy-hassinger\n",
      "leestafel.info/amy-hest-a-helen-oxenbury\n",
      "leestafel.info/an-leysen\n",
      "leestafel.info/an-swerts\n",
      "leestafel.info/an-vissers-a-christina-van-geel\n",
      "leestafel.info/ana-novac\n",
      "leestafel.info/anaid-haen-a-django-mathijsen\n",
      "leestafel.info/anand-hans\n",
      "leestafel.info/anchee-min\n",
      "leestafel.info/andre-boesberg\n",
      "leestafel.info/andre-donker\n",
      "leestafel.info/andre-en-nienke-nuyens\n",
      "leestafel.info/andre-kuipers\n",
      "leestafel.info/andre-van-teeffelen\n",
      "leestafel.info/andrea-badenoch\n",
      "leestafel.info/andrea-badenoch?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/andrea-badenoch?format=feed&type=atom\n",
      "leestafel.info/andrea-badenoch?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/andrea-badenoch?format=feed&type=rss\n",
      "leestafel.info/andrea-c-hoffmann-en-patience-i\n",
      "leestafel.info/andrea-eames\n",
      "leestafel.info/andrea-hanna-huenniger\n",
      "leestafel.info/andrea-rauch\n",
      "leestafel.info/andrea-voigt\n",
      "leestafel.info/andreas-rood\n",
      "leestafel.info/andreas-steinhoefel\n",
      "leestafel.info/andrew-davidson\n",
      "leestafel.info/andrew-graham--dixon\n",
      "leestafel.info/andrew-joyner\n",
      "leestafel.info/andrew-mccormick\n",
      "leestafel.info/andrew-pinder\n",
      "leestafel.info/andrew-solomon\n",
      "leestafel.info/andy-dennis-en-anna-simon\n",
      "leestafel.info/andy-griffiths-a-terry-denton\n",
      "leestafel.info/andy-harris\n",
      "leestafel.info/andy-mcdonald\n",
      "leestafel.info/angela-becerra\n",
      "leestafel.info/angela-becerra?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/angela-becerra?format=feed&type=atom\n",
      "leestafel.info/angela-becerra?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/angela-becerra?format=feed&type=rss\n",
      "leestafel.info/angela-crott\n",
      "leestafel.info/angela-dekker\n",
      "leestafel.info/angela-diterlizzi-a-brendan-wenzel\n",
      "leestafel.info/angela-flournoy\n",
      "leestafel.info/angeles-caso\n",
      "leestafel.info/angelique-van-dam\n",
      "leestafel.info/angelo-vergeer\n",
      "leestafel.info/angie-lake\n",
      "leestafel.info/aniet-c-bruininks-a-leo-m-harmsen\n",
      "leestafel.info/anila-wilms\n",
      "leestafel.info/anita-bijsterbosch\n",
      "leestafel.info/anita-boots\n",
      "leestafel.info/anita-brookner\n",
      "leestafel.info/anita-nair\n"
     ]
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "leestafel.info/anita-shreve\n",
      "leestafel.info/anita-van-den-bogaart\n",
      "leestafel.info/anja-van-biene\n",
      "leestafel.info/anja-vereijken\n",
      "leestafel.info/anke-de-jong-koele\n",
      "leestafel.info/anke-dorpmanns-a-maikel-verkoelen\n",
      "leestafel.info/anke-kranendonk\n",
      "leestafel.info/anke-kranendonk-romans\n",
      "leestafel.info/anke-scheeren\n",
      "leestafel.info/anke-werker-a-danielle-schothorst\n",
      "leestafel.info/ann-brashares\n",
      "leestafel.info/ann-de-bode\n",
      "leestafel.info/ann-downer\n",
      "leestafel.info/ann-lootens\n",
      "leestafel.info/anna-bikont\n",
      "leestafel.info/anna-brouwer\n",
      "leestafel.info/anna-chojnacka\n",
      "leestafel.info/anna-coudenys\n",
      "leestafel.info/anna-currey\n",
      "leestafel.info/anna-dale\n",
      "leestafel.info/anna-drijver\n",
      "leestafel.info/anna-enquist\n",
      "leestafel.info/anna-fredriksson\n",
      "leestafel.info/anna-kemp-a-sara-ogilvie\n",
      "leestafel.info/anna-lawrence-pietroni\n",
      "leestafel.info/anna-llenas\n",
      "leestafel.info/anna-mcpartlin\n",
      "leestafel.info/anna-nilsen\n",
      "leestafel.info/anna-pasternak\n",
      "leestafel.info/anna-van-praag\n",
      "leestafel.info/anna-woltz\n",
      "leestafel.info/annabel-pitcher\n",
      "leestafel.info/anne-baaths\n",
      "leestafel.info/anne-baaths?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/anne-baaths?format=feed&type=atom\n",
      "leestafel.info/anne-baaths?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/anne-baaths?format=feed&type=rss\n",
      "leestafel.info/anne-berry\n",
      "leestafel.info/anne-berry?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/anne-berry?format=feed&type=atom\n",
      "leestafel.info/anne-berry?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/anne-berry?format=feed&type=rss\n",
      "leestafel.info/anne-borsboom\n",
      "leestafel.info/anne-borsboom?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/anne-borsboom?format=feed&type=atom\n",
      "leestafel.info/anne-borsboom?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/anne-borsboom?format=feed&type=rss\n",
      "leestafel.info/anne-cassidy\n",
      "leestafel.info/anne-doedens\n",
      "leestafel.info/anne-enright\n",
      "leestafel.info/anne-haverty\n",
      "leestafel.info/anne-karin-elstad\n",
      "leestafel.info/anne-keijzer\n",
      "leestafel.info/anne-laure-bondoux\n",
      "leestafel.info/anne-margot-ramstein-a-matthias-aregui\n",
      "leestafel.info/anne-marie-hoekstra\n",
      "leestafel.info/anne-marie-v-cappelle\n",
      "leestafel.info/anne-marie-van-cappelle-7-9-jr\n",
      "leestafel.info/anne-marie-van-der-wilt\n",
      "leestafel.info/anne-marieke-samson\n",
      "leestafel.info/anne-neijzen\n",
      "leestafel.info/anne-nivat\n",
      "leestafel.info/anne-provoost-10-12-jr\n",
      "leestafel.info/anne-sawan\n",
      "leestafel.info/anne-schneider\n",
      "leestafel.info/anne-sward\n",
      "leestafel.info/anne-van-der-dool\n",
      "leestafel.info/anne-voorhoeve\n",
      "leestafel.info/anne-wesseling\n",
      "leestafel.info/anne-west\n",
      "leestafel.info/annegreet-van-bergen\n",
      "leestafel.info/annejet-van-der-zijl\n",
      "leestafel.info/annejoke-smids\n",
      "leestafel.info/anneke-de-bundel-a-nicole-franken\n",
      "leestafel.info/anneke-dok\n",
      "leestafel.info/anneke-scholtens\n",
      "leestafel.info/annelies-aa-vanbelle\n",
      "leestafel.info/annelies-beck\n",
      "leestafel.info/annelies-beck?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/annelies-beck?format=feed&type=atom\n",
      "leestafel.info/annelies-beck?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/annelies-beck?format=feed&type=rss\n",
      "leestafel.info/annelies-verbeke\n",
      "leestafel.info/anneloes-timmerije\n",
      "leestafel.info/annemarie-bleeker\n",
      "leestafel.info/annemarie-bleeker?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/annemarie-bleeker?format=feed&type=atom\n",
      "leestafel.info/annemarie-bleeker?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/annemarie-bleeker?format=feed&type=rss\n",
      "leestafel.info/annemarie-bon\n",
      "leestafel.info/annemarie-bon-a-marie-jose-van-der-linden\n",
      "leestafel.info/annemarie-van-den-brink\n",
      "leestafel.info/annemarie-van-der-heijden\n",
      "leestafel.info/annemarie-van-haeringen\n",
      "leestafel.info/annemerel-de-jongh\n",
      "leestafel.info/annemie-berebrouckx\n",
      "leestafel.info/annemie-vandaele\n",
      "leestafel.info/annemiek-schrijver\n",
      "leestafel.info/annemiek-van-kessel\n",
      "leestafel.info/annemieke-houben\n",
      "leestafel.info/annemieke-pecht\n",
      "leestafel.info/anneriek-v-heugten\n",
      "leestafel.info/anneriek-van-heugten\n",
      "leestafel.info/annet-huizing\n",
      "leestafel.info/annet-jacob\n",
      "leestafel.info/annette-zeelenberg\n",
      "leestafel.info/annick-masson\n",
      "leestafel.info/annie-mg-schmidt\n",
      "leestafel.info/annie-schreuders-derks\n",
      "leestafel.info/annie-van-gansewinkel\n",
      "leestafel.info/annie-van-keymeulen\n",
      "leestafel.info/anniek-winters\n",
      "leestafel.info/anny-duperey\n",
      "leestafel.info/anoniem\n",
      "leestafel.info/anoniem?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/anoniem?format=feed&type=atom\n",
      "leestafel.info/anoniem?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/anoniem?format=feed&type=rss\n",
      "leestafel.info/anouck-boisrobert\n",
      "leestafel.info/anouk-brusse\n",
      "leestafel.info/ans-van-melis\n",
      "leestafel.info/anthonie-van-de-wardt\n",
      "leestafel.info/anthony-doerr\n",
      "leestafel.info/anthony-gottlieb\n",
      "leestafel.info/antje-visser\n",
      "leestafel.info/antoine-bodar\n",
      "leestafel.info/antoine-rault\n",
      "leestafel.info/anton-de-kom\n",
      "leestafel.info/anton-quintana\n",
      "leestafel.info/anton-tsjechov\n",
      "leestafel.info/anton-valens\n",
      "leestafel.info/antoni-libera\n",
      "leestafel.info/antonia-arslan\n",
      "leestafel.info/antonia-arslan?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/antonia-arslan?format=feed&type=atom\n",
      "leestafel.info/antonia-arslan?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/antonia-arslan?format=feed&type=rss\n",
      "leestafel.info/antonia-fraser\n",
      "leestafel.info/antonia-michaelis\n",
      "leestafel.info/antonio-forcellino\n",
      "leestafel.info/antonio-skarmeta\n",
      "leestafel.info/antonio-tabucchi\n",
      "leestafel.info/antony-penrose\n",
      "leestafel.info/ap-herbert\n",
      "leestafel.info/ap-van-der-meulen\n",
      "leestafel.info/apostolos-doxiadis\n",
      "leestafel.info/aravind-adiga\n",
      "leestafel.info/aravind-adiga?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/aravind-adiga?format=feed&type=atom\n",
      "leestafel.info/aravind-adiga?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/aravind-adiga?format=feed&type=rss\n",
      "leestafel.info/ard-volkers\n",
      "leestafel.info/arend-jan-heerma-van-voss\n",
      "leestafel.info/arend-van-dam\n",
      "leestafel.info/arianne-grootendorst\n",
      "leestafel.info/arie-boomsma\n",
      "leestafel.info/arie-boomsma?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/arie-boomsma?format=feed&type=atom\n",
      "leestafel.info/arie-boomsma?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/arie-boomsma?format=feed&type=rss\n",
      "leestafel.info/arie-storm\n",
      "leestafel.info/arie-van-der-veer-a-ellen-laninga\n",
      "leestafel.info/ariella-kornmehl\n",
      "leestafel.info/arienne-bolt\n",
      "leestafel.info/aristide-von-bienefeldt\n",
      "leestafel.info/aristide-von-bienefeldt?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/aristide-von-bienefeldt?format=feed&type=atom\n",
      "leestafel.info/aristide-von-bienefeldt?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/aristide-von-bienefeldt?format=feed&type=rss\n",
      "leestafel.info/arita-baaijens\n",
      "leestafel.info/arja-veerman\n",
      "leestafel.info/arjaan-van-nimwegen\n",
      "leestafel.info/arjan-visser\n",
      "leestafel.info/arjen-lubach\n",
      "leestafel.info/arjen-mulder\n",
      "leestafel.info/arjen-terpstra\n",
      "leestafel.info/arne-zuidhoek\n",
      "leestafel.info/arno-bohlmeijer\n",
      "leestafel.info/arno-haijtema\n",
      "leestafel.info/arno-wubben\n",
      "leestafel.info/arnout-en-nico-guns\n",
      "leestafel.info/aron-brouwer-en-marthijn-wouters\n",
      "leestafel.info/aron-dijkstra\n",
      "leestafel.info/arthur-stockwin\n",
      "leestafel.info/arthur-umbgrove\n",
      "leestafel.info/arthur-wevers\n",
      "leestafel.info/arundhati-roy\n",
      "leestafel.info/arvo-valton\n",
      "leestafel.info/aryn-kyle\n",
      "leestafel.info/ashild-kanstad-johnsen\n",
      "leestafel.info/asis-aynan\n",
      "leestafel.info/asis-aynan?format=feed&type=atom\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/asis-aynan?format=feed&type=atom\n",
      "leestafel.info/asis-aynan?format=feed&type=rss\n",
      "'NoneType' object has no attribute 'find_all'\n",
      "Skipping leestafel.info/asis-aynan?format=feed&type=rss\n",
      "leestafel.info/asne-seierstad\n",
      "leestafel.info/astrid-harrewijn\n",
      "leestafel.info/astrid-lindgren\n"
     ]
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "leestafel.info/astrid-sy\n",
      "leestafel.info/astrid-witte\n",
      "leestafel.info/ate-hoekstra\n",
      "leestafel.info/atticus-lish\n",
      "leestafel.info/audrey-niffenegger\n",
      "leestafel.info/august-willemsen\n",
      "leestafel.info/augusta-verburg\n",
      "leestafel.info/augusta-verburg-4-6-jr\n",
      "leestafel.info/augusto-cruz\n",
      "leestafel.info/auke-van-stralen\n",
      "leestafel.info/aurora-marsotto\n",
      "leestafel.info/ayana-mathis\n",
      "leestafel.info/ayano-imai\n",
      "leestafel.info/ayu-utami\n"
     ]
    }
   ],
   "source": [
    "import re\n",
    "import json\n",
    "import copy\n",
    "from bs4 import BeautifulSoup\n",
    "\n",
    "class LeestafelParser(object):\n",
    "    \n",
    "    def __init__(self, oclc_api_key):\n",
    "        self.oclc = OCLC_API(oclc_api_key)\n",
    "    \n",
    "    def is_review(self, article):\n",
    "        if self.has_title_header(article):\n",
    "            return True\n",
    "        return False\n",
    "\n",
    "    def has_title_header(self, para):\n",
    "        titles_soup = para.find_all(class_=\"article_heading1\")\n",
    "        if titles_soup:\n",
    "            return True\n",
    "        return False\n",
    "\n",
    "    def get_title_info(self, para, review):\n",
    "        review[\"title\"] = self.get_title(para)\n",
    "        for author in self.get_author(para):\n",
    "            # add any authors beyond the main author that the page is about\n",
    "            if author not in review[\"author\"]:\n",
    "                review[\"author\"].append(author)\n",
    "\n",
    "    def get_title(self, para):\n",
    "        titles_soup = para.find_all(class_=\"article_heading1\")\n",
    "        for title_soup in titles_soup:\n",
    "            if title_soup.text == \"\":\n",
    "                continue\n",
    "            return title_soup.text\n",
    "            break\n",
    "        return None\n",
    "\n",
    "    def get_author(self, content_soup):\n",
    "        authors = []\n",
    "        try:\n",
    "            author_soup = content_soup.find_all(class_=\"componentheading\")[0]\n",
    "            return [author_soup.text] # return as list, may have co-authors on certain reviewed books\n",
    "        except AttributeError:\n",
    "            return []\n",
    "        except IndexError:\n",
    "            pass\n",
    "        authors_soup = content_soup.find_all(class_=\"article_heading2\")\n",
    "        if not authors_soup:\n",
    "            return []\n",
    "        for author_soup in authors_soup:\n",
    "            if author_soup.text != \"\" and author_soup.text != \" \":\n",
    "                authors += [author_string for author_string in author_soup.stripped_strings]\n",
    "        return authors\n",
    "\n",
    "    def has_messageboard_link(self, para):\n",
    "        links = para.find_all('a')\n",
    "        if not links:\n",
    "            return False\n",
    "        for link in links:\n",
    "            if \"http://leestafel.messageboard.nl/forum/viewtopic.php?\" in link['href']:\n",
    "                return True\n",
    "        return False\n",
    "\n",
    "    def normalize_isbn(self, text):\n",
    "        text = re.sub(r\"ISBN(10|13)?:?\", r\"ISBN\", text)\n",
    "        if \"ISBN 978\" in text:\n",
    "            while re.match(r\"ISBN [0-9]{,8} [0-9]\", text):\n",
    "                text = re.sub(r\"ISBN ([0-9]{,11}) +([0-9])\", r\"ISBN \\1\\2\", text)\n",
    "        else:\n",
    "            while re.match(r\"ISBN [0-9]{,8} [0-9]\", text):\n",
    "                text = re.sub(r\"ISBN ([0-9]{,8}) +([0-9])\", r\"ISBN \\1\\2\", text)\n",
    "        return text\n",
    "\n",
    "    def get_isbn(self, text, review):\n",
    "        text = self.normalize_isbn(text)\n",
    "        m = re.search(\"ISBN ([0-9Xx]{10,13})\", text)\n",
    "        review[\"isbn\"] = [m.group(1)]\n",
    "\n",
    "    def has_isbn(self, text):\n",
    "        if \"ISBN\" in text:\n",
    "            text = self.normalize_isbn(text)\n",
    "        m = re.search(\"ISBN(10|13)?:? ([0-9Xx]{10,13})\", text)\n",
    "        return True if m else False\n",
    "\n",
    "    def has_reviewer_metadata(self, text):\n",
    "        if r\"©\" in text:\n",
    "            return True\n",
    "        return False\n",
    "    \n",
    "    def extract_review_date(self, copyright, review):\n",
    "        months = \"\"\n",
    "        m = re.search(r\"((\\d{1,2} +)?(januari|februari|maart|april|mei|juni|juli|augustus|september|oktober|november|december) +(\\d{4}))\", copyright)\n",
    "        if m:\n",
    "            review[\"review_date\"] = m.group(0)\n",
    "            copyright = re.sub(m.group(0), \"\", copyright)\n",
    "            copyright = re.sub(\"[,\\. ]+$\", \"\", copyright)\n",
    "        return copyright\n",
    "\n",
    "    def get_reviewer_metadata(self, text, review):\n",
    "        # Assumption: reviewer signs off with copyright symbol, followed by name, followed by date\n",
    "        # Example: © Alice, 23-05-2011\n",
    "        copyright = re.sub(r\".*©\", \"©\", text).replace(\"©,\",\"©\").replace(\"\\n\",\"\")\n",
    "        #copyright = copyright\n",
    "        copyright = self.extract_review_date(copyright, review)\n",
    "        review[\"reviewer\"] = re.sub(\"© +\",\"\", copyright).strip()\n",
    "\n",
    "    def get_articles(self, content_soup):\n",
    "        return content_soup.find_all(class_=\"article_column\")\n",
    "\n",
    "    def get_review(self, author, article_soup):\n",
    "        review = { \"author\": copy.copy(author), \"review_text\": \"\", \"unparsed_metadata\": [] }\n",
    "        if self.has_title_header(article_soup):\n",
    "            self.get_title_info(article_soup, review)\n",
    "        self.status = \"text\"\n",
    "        for p in article_soup.find_all('p'):\n",
    "            self.parse_paragraph(p, review)\n",
    "        if \"title\" not in review:\n",
    "            return None\n",
    "        if \"isbn\" not in review.keys():\n",
    "            review[\"isbn\"] = self.lookup_isbn(review[\"author\"][0], review[\"title\"])\n",
    "        return review\n",
    "\n",
    "    def lookup_isbn(self, author, title):\n",
    "        editions = self.oclc.find_work_metadata(author, title)\n",
    "        return [edition[\"isbn\"] for edition in editions if \"isbn\" in edition]\n",
    "\n",
    "    def parse_paragraph(self, p, review):\n",
    "        paragraph = p.get_text().replace(\"\\u00a0\", \" \") # remobe non-breaking spaces\n",
    "        if self.has_title_header(p):\n",
    "            return True\n",
    "        if self.has_messageboard_link(p):\n",
    "            return True\n",
    "        if self.has_reviewer_metadata(paragraph):\n",
    "            self.get_reviewer_metadata(paragraph, review)\n",
    "        if self.has_isbn(paragraph):\n",
    "            self.status = \"metadata\" # from here all paragraphs are metadata\n",
    "            self.get_isbn(paragraph, review)\n",
    "        if self.status == \"metadata\":\n",
    "            review[\"unparsed_metadata\"].append(paragraph)\n",
    "        else:\n",
    "            review[\"review_text\"] += \" \" + paragraph\n",
    "    \n",
    "    def get_content(self, fname):\n",
    "        with open(fname, 'rt') as fh:\n",
    "            soup = BeautifulSoup(fh, \"lxml\")\n",
    "            return soup.find(id='main')\n",
    "\n",
    "    def parse_review_page(self, fname):\n",
    "        content_soup = self.get_content(fname)\n",
    "        author = self.get_author(content_soup)\n",
    "        articles_soup = self.get_articles(content_soup)\n",
    "        if not articles_soup: # no review articles found\n",
    "            return []\n",
    "        reviews = [self.get_review(author, article_soup) for article_soup in articles_soup]\n",
    "        return [review for review in reviews if review] # skip null or None\n",
    "\n",
    "site = \"leestafel.info\"\n",
    "files = !ls {site}/a*\n",
    "#print(files)\n",
    "\n",
    "oclc_api_key_file = \"oclc-api-key\"\n",
    "with open(oclc_api_key_file, 'rt') as fh:\n",
    "    oclc_api_key = fh.read().strip()\n",
    "\n",
    "leestafel_parser = LeestafelParser(oclc_api_key)\n",
    "\n",
    "output_file = \"leestafel.a.json\"\n",
    "\n",
    "with open(output_file, 'wt') as fh:\n",
    "    for fname in files:\n",
    "        print(fname)\n",
    "        try:\n",
    "            reviews = leestafel_parser.parse_review_page(fname)\n",
    "            fh.write(json.dumps(reviews) + \"\\n\")\n",
    "        except AttributeError as err: # catch all for Beautiful soup errors, need to improve\n",
    "            print(err)\n",
    "            print(\"Skipping\", fname)\n",
    "            pass\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "collapsed": true
   },
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.2"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 1
}
